@@ -123,35 +123,48 @@ class IPAddresses():
 last_location = Gio.File.new_for_path(GLib.get_home_dir())
 # A normal GtkFileChooserDialog only lets you pick folders OR files, not
 # both in the same dialog.  This does.
+
+class FolderFileChooserDialog(Gtk.Dialog):
+    def __init__(self, window_title, transient_parent, starting_location):
+        super(FolderFileChooserDialog, self).__init__(title=window_title,
+                                                      parent=transient_parent,
+                                                      default_width=750,
+                                                      default_height=500)
+
+        self.add_buttons(_("Cancel"), Gtk.ResponseType.CANCEL,
+                         _("Add"), Gtk.ResponseType.OK)
+
+        self.chooser = Gtk.FileChooserWidget(action=Gtk.FileChooserAction.OPEN, select_multiple=True)
+        self.chooser.set_current_folder_file(starting_location)
+        self.chooser.connect("file-activated", lambda chooser: self.response(Gtk.ResponseType.OK))
+        self.chooser.show_all()
+
+        self.get_content_area().add(self.chooser)
+        self.get_content_area().set_border_width(0)
+        self.get_uris = self.chooser.get_uris
+        self.get_current_folder_file = self.chooser.get_current_folder_file
+        self.connect("key-press-event", self.on_button_press)
+
+    def on_button_press(self, widget, event, data=None):
+        multi = len(self.chooser.get_uris()) != 1
+        if event.keyval in (Gdk.KEY_KP_Enter, Gdk.KEY_Return) and multi:
+            self.response(Gtk.ResponseType.OK)
+            return Gdk.EVENT_STOP
+
+        return Gdk.EVENT_PROPAGATE
+
 def create_file_and_folder_picker(dialog_parent=None):
-    window = Gtk.Dialog(title=_("Select file(s) to send"),
-                        parent=dialog_parent,
-                        default_width=750,
-                        default_height=500)
-    window.add_buttons(_("Cancel"), Gtk.ResponseType.CANCEL,
-                       _("Send"), Gtk.ResponseType.ACCEPT)
-
-    chooser = Gtk.FileChooserWidget(action=Gtk.FileChooserAction.OPEN,
-                                    select_multiple=True)
-
-    chooser.set_current_folder_file(last_location)
-    chooser.connect("file-activated", lambda chooser: window.response(Gtk.ResponseType.ACCEPT))
+    chooser = FolderFileChooserDialog(_("Select file(s) to send"), dialog_parent, last_location)
 
     def update_last_location(dialog, response_id, data=None):
-        if response_id != Gtk.ResponseType.ACCEPT:
+        if response_id != Gtk.ResponseType.OK:
             return
 
         global last_location
         last_location = chooser.get_current_folder_file()
 
-    window.connect("response", update_last_location)
-
-    chooser.show_all()
-    window.get_content_area().add(chooser)
-    window.get_content_area().set_border_width(0)
-
-    window.get_uris = chooser.get_uris
-    return window
+    chooser.connect("response", update_last_location)
+    return chooser
 
 # Used as a decorator to run things in the background
 def _async(func):
